@@ -7,9 +7,10 @@
 #line 4 "vte.override"
 #include <Python.h>
 #include <pygtk/pygtk.h>
+#include <pygobject.h>
 #include <gtk/gtk.h>
 #include "../src/vte.h"
-#line 13 "vte.c"
+#line 14 "vte.c"
 
 
 /* ---------- types from other modules ---------- */
@@ -46,21 +47,26 @@ _wrap_vte_terminal_new(PyGObject *self, PyObject *args, PyObject *kwargs)
     return 0;
 }
 
-#line 17 "vte.override"
+#line 15 "vte.override"
+
 static PyObject *
 _wrap_vte_terminal_fork_command(PyGObject * self, PyObject * args,
 				PyObject * kwargs)
 {
 	gchar **argv = NULL, **envv = NULL;
-	gchar *command = NULL;
-	static char *kwlist[] = { "command", "argv", "envv", NULL };
-	PyObject *py_argv = NULL, *py_envv = NULL;
+	gchar *command = NULL, *directory = NULL;
+	static char *kwlist[] = { "command", "argv", "envv", "directory",
+				  "loglastlog", "logutmp", "logwtmp",
+				  NULL };
+	PyObject *py_argv = NULL, *py_envv = NULL,
+		 *loglastlog = NULL, *logutmp = NULL, *logwtmp = NULL;
 	int i, n_args, n_envs;
 	pid_t pid;
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|sOO:fork_command",
-					 kwlist, &command, &py_argv,
-					 &py_envv)) {
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|sOOsOOO:fork_command",
+					 kwlist, &command, &py_argv, &py_envv,
+					 &directory,
+					 &loglastlog, &logutmp, &logwtmp)) {
 		return NULL;
 	}
 
@@ -100,7 +106,13 @@ _wrap_vte_terminal_fork_command(PyGObject * self, PyObject * args,
 	}
 
 	pid = vte_terminal_fork_command(VTE_TERMINAL(self->obj),
-					command, argv, envv);
+					command, argv, envv, directory,
+					(loglastlog != NULL) &&
+					PyObject_IsTrue(loglastlog),
+					(logutmp != NULL) &&
+					PyObject_IsTrue(logutmp),
+					(logwtmp != NULL) &&
+					PyObject_IsTrue(logwtmp));
 
 	if (argv) {
 		g_free(argv);
@@ -108,7 +120,7 @@ _wrap_vte_terminal_fork_command(PyGObject * self, PyObject * args,
 
 	return PyInt_FromLong(pid);
 }
-#line 112 "vte.c"
+#line 124 "vte.c"
 
 
 static PyObject *
@@ -314,15 +326,16 @@ _wrap_vte_terminal_set_color_background(PyGObject *self, PyObject *args, PyObjec
     return Py_None;
 }
 
+#line 308 "vte.override"
 static PyObject *
 _wrap_vte_terminal_set_colors(PyGObject *self, PyObject *args, PyObject *kwargs)
 {
-    static char *kwlist[] = { "foreground", "background", "palette", "palette_size", NULL };
-    PyObject *py_foreground, *py_background, *py_palette;
-    int palette_size;
+    static char *kwlist[] = { "foreground", "background", "palette", NULL };
+    PyObject *py_foreground, *py_background, *py_palette, *item;
+    int palette_size, i;
     GdkColor *foreground = NULL, *background = NULL, *palette = NULL;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "OOOi:VteTerminal.set_colors", kwlist, &py_foreground, &py_background, &py_palette, &palette_size))
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "OOO:VteTerminal.set_colors", kwlist, &py_foreground, &py_background, &py_palette, &palette_size))
         return NULL;
     if (pyg_boxed_check(py_foreground, GDK_TYPE_COLOR))
         foreground = pyg_boxed_get(py_foreground, GdkColor);
@@ -336,16 +349,30 @@ _wrap_vte_terminal_set_colors(PyGObject *self, PyObject *args, PyObject *kwargs)
         PyErr_SetString(PyExc_TypeError, "background should be a GdkColor");
         return NULL;
     }
-    if (pyg_boxed_check(py_palette, GDK_TYPE_COLOR))
-        palette = pyg_boxed_get(py_palette, GdkColor);
-    else {
-        PyErr_SetString(PyExc_TypeError, "palette should be a GdkColor");
+    if (PySequence_Check(py_palette)) {
+	palette_size = PySequence_Length(py_palette);
+        palette = g_malloc(sizeof(GdkColor) * palette_size);
+	for (i = 0; i < palette_size; i++) {
+	    item = PySequence_GetItem(py_palette, i);
+            if (!pyg_boxed_check(item, GDK_TYPE_COLOR)) {
+		g_free(palette);
+        	PyErr_SetString(PyExc_TypeError, "palette should be a list of GdkColors");
+		return NULL;
+	    }
+	    palette[i] = *((GdkColor*)pyg_boxed_get(py_palette, GdkColor));
+            Py_DECREF(item);
+	}
+    } else {
+        PyErr_SetString(PyExc_TypeError, "palette should be a list of GdkColors");
         return NULL;
     }
     vte_terminal_set_colors(VTE_TERMINAL(self->obj), foreground, background, palette, palette_size);
+    g_free(palette);
     Py_INCREF(Py_None);
     return Py_None;
 }
+#line 375 "vte.c"
+
 
 static PyObject *
 _wrap_vte_terminal_set_default_colors(PyGObject *self)
@@ -498,6 +525,28 @@ _wrap_vte_terminal_get_using_xft(PyGObject *self)
     return PyInt_FromLong(ret);
 }
 
+static PyObject *
+_wrap_vte_terminal_set_allow_bold(PyGObject *self, PyObject *args, PyObject *kwargs)
+{
+    static char *kwlist[] = { "allow_bold", NULL };
+    int allow_bold;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "i:VteTerminal.set_allow_bold", kwlist, &allow_bold))
+        return NULL;
+    vte_terminal_set_allow_bold(VTE_TERMINAL(self->obj), allow_bold);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *
+_wrap_vte_terminal_get_allow_bold(PyGObject *self)
+{
+    int ret;
+
+    ret = vte_terminal_get_allow_bold(VTE_TERMINAL(self->obj));
+    return PyInt_FromLong(ret);
+}
+
 static PyObject *
 _wrap_vte_terminal_get_has_selection(PyGObject *self)
 {
@@ -606,6 +655,229 @@ _wrap_vte_terminal_reset(PyGObject *self, PyObject *args, PyObject *kwargs)
     return Py_None;
 }
 
+#line 89 "vte.override"
+
+static gboolean
+call_callback(VteTerminal *terminal, glong column, glong row, gpointer data)
+{
+    PyObject *cb, *self, *args, *result;
+    gboolean ret;
+    int i;
+    if (!PyArg_ParseTuple(data, "|O:call_callback", &args)) {
+        return FALSE;
+    }
+    args = PyList_New(0);
+    cb = PySequence_GetItem(args, 0); /* INCREFs */
+    Py_DECREF(cb);
+    self = PySequence_GetItem(args, 1); /* INCREFs */
+    Py_DECREF(self);
+    PyList_Append(args, self);
+    PyList_Append(args, PyInt_FromLong(column));
+    PyList_Append(args, PyInt_FromLong(row));
+    for (i = 2; i < PySequence_Length(args); i++) {
+        PyObject *item = PySequence_GetItem(args, i);
+	Py_DECREF(item);
+        PyList_Append(args, item);
+    }
+    result = PyObject_CallObject(cb, args);
+    ret = (result && PyObject_IsTrue(result));
+    Py_DECREF(args);
+    Py_DECREF(result);
+    return ret;
+}
+
+static gboolean
+always_true(VteTerminal *terminal, glong row, glong column, gpointer data)
+{
+    return TRUE;
+}
+static PyObject *
+_wrap_vte_terminal_get_text(PyGObject *self, PyObject *args, PyObject *kwargs)
+{
+    static char *kwlist[] = { "callback", "attributes", "data", NULL };
+    PyObject *callback = NULL, *do_attr = NULL, *data = NULL;
+    PyObject *callback_and_args = NULL;
+    GArray *attrs = NULL;
+    PyObject *text;
+    PyObject *py_attrs;
+    int count;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|OOO:terminal_get_text",
+				     kwlist, &callback, &do_attr, &args)) {
+        return NULL;
+    }
+
+    if (do_attr != NULL && do_attr != Py_None) {
+	attrs = g_array_new(FALSE, TRUE, sizeof(struct vte_char_attributes));
+    } else {
+    	attrs = NULL;
+    }
+
+    if ((callback != NULL) && (callback != Py_None)) {
+	if (!PyCallable_Check(callback)) {
+	    PyErr_SetString(PyExc_TypeError, "1st argument not callable.");
+            if (attrs) {
+                g_array_free(attrs, TRUE);
+            }
+	    return NULL;
+	}
+    } else {
+    	callback = NULL;
+    }
+
+    if (callback != NULL) {
+        callback_and_args = PyList_New(0);
+        PyList_Append(callback_and_args, callback);
+        PyList_Append(callback_and_args, (PyObject*) self);
+        if (PyList_Check(data)) {
+            for (count = 0; count < PyList_Size(data); count++) {
+                PyList_Append(callback_and_args, PyList_GetItem(data, count));
+            }
+        } else {
+            PyList_Append(callback_and_args, data);
+        }
+    }
+
+    text = PyString_FromString(vte_terminal_get_text(VTE_TERMINAL(self->obj),
+						     callback ?
+						     call_callback :
+						     always_true,
+						     callback ?
+						     callback_and_args :
+						     NULL,
+						     attrs));
+    Py_XDECREF(callback_and_args);
+
+    if (attrs) {
+	py_attrs = PyTuple_New(attrs->len);
+	for (count = 0; count < attrs->len; count++) {
+	    struct vte_char_attributes *cht;
+	    PyObject *py_char_attr;
+	    PyObject *py_gdkcolor;
+
+	    cht = &g_array_index(attrs, struct vte_char_attributes, count);
+	    py_char_attr = PyDict_New();
+	    PyDict_SetItemString(py_char_attr, "row", PyInt_FromLong(cht->row));
+	    PyDict_SetItemString(py_char_attr, "column", PyInt_FromLong(cht->column));
+	    
+	    py_gdkcolor = pyg_boxed_new(GDK_TYPE_COLOR, &cht->fore, TRUE, TRUE);
+	    PyDict_SetItemString(py_char_attr, "fore", py_gdkcolor);
+	    py_gdkcolor = pyg_boxed_new(GDK_TYPE_COLOR, &cht->back, TRUE, TRUE);
+	    PyDict_SetItemString(py_char_attr, "back", py_gdkcolor);
+	    
+	    PyDict_SetItemString(py_char_attr, "underline", 
+		    PyInt_FromLong(cht->underline));
+	    PyDict_SetItemString(py_char_attr, "strikethrough", 
+		    PyInt_FromLong(cht->strikethrough));
+
+	    PyTuple_SetItem(py_attrs, count, py_char_attr);
+	}
+	g_array_free(attrs, TRUE);
+	return Py_BuildValue("(OO)", text, py_attrs);
+    } else {
+    	return Py_BuildValue("O", text);
+    }
+}
+#line 782 "vte.c"
+
+
+#line 213 "vte.override"
+static PyObject *
+_wrap_vte_terminal_get_text_range(PyGObject *self, PyObject *args, PyObject *kwargs)
+{
+    static char *kwlist[] = { "callback", "start_row", "start_col", "end_row",
+			      "end_col", "attributes", "data", NULL };
+    PyObject *callback = NULL, *do_attr = NULL, *data = NULL;
+    glong start_row, start_col, end_row, end_col;
+    PyObject *callback_and_args = NULL;
+    GArray *attrs = NULL;
+    PyObject *text;
+    PyObject *py_attrs;
+    int count;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|llllOOO:terminal_get_text",
+				     kwlist,
+				     &start_row, &start_col, &end_row, &end_col,
+				     &callback, &do_attr, &args)) {
+        return NULL;
+    }
+
+    if (do_attr != NULL && do_attr != Py_None) {
+	attrs = g_array_new(FALSE, TRUE, sizeof(struct vte_char_attributes));
+    } else {
+    	attrs = NULL;
+    }
+
+    if ((callback != NULL) && (callback != Py_None)) {
+	if (!PyCallable_Check(callback)) {
+	    PyErr_SetString(PyExc_TypeError, "1st argument not callable.");
+            if (attrs) {
+                g_array_free(attrs, TRUE);
+            }
+	    return NULL;
+	}
+    } else {
+    	callback = NULL;
+    }
+
+    if (callback != NULL) {
+        callback_and_args = PyList_New(0);
+        PyList_Append(callback_and_args, callback);
+        PyList_Append(callback_and_args, (PyObject*) self);
+        if (PyList_Check(data)) {
+            for (count = 0; count < PyList_Size(data); count++) {
+                PyList_Append(callback_and_args, PyList_GetItem(data, count));
+            }
+        } else {
+            PyList_Append(callback_and_args, data);
+        }
+    }
+
+    text = PyString_FromString(vte_terminal_get_text_range(VTE_TERMINAL(self->obj),
+							   start_row, start_col,
+							   end_row, end_col,
+							   callback ?
+							   call_callback :
+							   always_true,
+							   callback ?
+							   callback_and_args :
+							   NULL,
+							   attrs));
+    Py_XDECREF(callback_and_args);
+
+    if (attrs) {
+	py_attrs = PyTuple_New(attrs->len);
+	for (count = 0; count < attrs->len; count++) {
+	    struct vte_char_attributes *cht;
+	    PyObject *py_char_attr;
+	    PyObject *py_gdkcolor;
+
+	    cht = &g_array_index(attrs, struct vte_char_attributes, count);
+	    py_char_attr = PyDict_New();
+	    PyDict_SetItemString(py_char_attr, "row", PyInt_FromLong(cht->row));
+	    PyDict_SetItemString(py_char_attr, "column", PyInt_FromLong(cht->column));
+	    
+	    py_gdkcolor = pyg_boxed_new(GDK_TYPE_COLOR, &cht->fore, TRUE, TRUE);
+	    PyDict_SetItemString(py_char_attr, "fore", py_gdkcolor);
+	    py_gdkcolor = pyg_boxed_new(GDK_TYPE_COLOR, &cht->back, TRUE, TRUE);
+	    PyDict_SetItemString(py_char_attr, "back", py_gdkcolor);
+	    
+	    PyDict_SetItemString(py_char_attr, "underline", 
+		    PyInt_FromLong(cht->underline));
+	    PyDict_SetItemString(py_char_attr, "strikethrough", 
+		    PyInt_FromLong(cht->strikethrough));
+
+	    PyTuple_SetItem(py_attrs, count, py_char_attr);
+	}
+	g_array_free(attrs, TRUE);
+	return Py_BuildValue("(OO)", text, py_attrs);
+    } else {
+    	return Py_BuildValue("O", text);
+    }
+}
+#line 879 "vte.c"
+
+
 static PyObject *
 _wrap_vte_terminal_match_clear_all(PyGObject *self)
 {
@@ -808,6 +1080,8 @@ static PyMethodDef _PyVteTerminal_methods[] = {
     { "set_font_from_string", (PyCFunction)_wrap_vte_terminal_set_font_from_string, METH_VARARGS|METH_KEYWORDS },
     { "get_font", (PyCFunction)_wrap_vte_terminal_get_font, METH_NOARGS },
     { "get_using_xft", (PyCFunction)_wrap_vte_terminal_get_using_xft, METH_NOARGS },
+    { "set_allow_bold", (PyCFunction)_wrap_vte_terminal_set_allow_bold, METH_VARARGS|METH_KEYWORDS },
+    { "get_allow_bold", (PyCFunction)_wrap_vte_terminal_get_allow_bold, METH_NOARGS },
     { "get_has_selection", (PyCFunction)_wrap_vte_terminal_get_has_selection, METH_NOARGS },
     { "set_word_chars", (PyCFunction)_wrap_vte_terminal_set_word_chars, METH_VARARGS|METH_KEYWORDS },
     { "is_word_char", (PyCFunction)_wrap_vte_terminal_is_word_char, METH_VARARGS|METH_KEYWORDS },
@@ -816,6 +1090,8 @@ static PyMethodDef _PyVteTerminal_methods[] = {
     { "set_mouse_autohide", (PyCFunction)_wrap_vte_terminal_set_mouse_autohide, METH_VARARGS|METH_KEYWORDS },
     { "get_mouse_autohide", (PyCFunction)_wrap_vte_terminal_get_mouse_autohide, METH_NOARGS },
     { "reset", (PyCFunction)_wrap_vte_terminal_reset, METH_VARARGS|METH_KEYWORDS },
+    { "get_text", (PyCFunction)_wrap_vte_terminal_get_text, METH_VARARGS|METH_KEYWORDS },
+    { "get_text_range", (PyCFunction)_wrap_vte_terminal_get_text_range, METH_VARARGS|METH_KEYWORDS },
     { "match_clear_all", (PyCFunction)_wrap_vte_terminal_match_clear_all, METH_NOARGS },
     { "match_add", (PyCFunction)_wrap_vte_terminal_match_add, METH_VARARGS|METH_KEYWORDS },
     { "set_emulation", (PyCFunction)_wrap_vte_terminal_set_emulation, METH_VARARGS|METH_KEYWORDS },
@@ -938,6 +1214,6 @@ pyvte_register_classes(PyObject *d)
     }
 
 
-#line 942 "vte.c"
+#line 1218 "vte.c"
     pygobject_register_class(d, "VteTerminal", VTE_TYPE_TERMINAL, &PyVteTerminal_Type, Py_BuildValue("(O)", &PyGtkWidget_Type));
 }
